#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Laguerre implementation
    /// </summary>
    [Description("Laguerre implementation")]
    [Gui.Design.DisplayName("Laguerre")]
    public class Laguerre : Indicator
    {
        #region Variables
        // Wizard generated variables
            private double gamma = 0.550; // Default setting for Gamma
        // User defined variables (add any user defined variables below)
			private double L0 = 0;
			private double L1 = 0;
			private double L2 = 0;
			private double L3 = 0;
			private double L0A = 0;
			private double L1A = 0;
			private double L2A = 0;
			private double L3A = 0;
			private double LRSI = 0;
			private double CU = 0;
			private double CD = 0;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "Plot0"));
			Add(new Line(Color.Gray, 0.1, "0.10"));
			Add(new Line(Color.Gray, 0.15, "0.15"));
			Add(new Line(Color.Gray, 0.85, "0.85"));
			Add(new Line(Color.Gray, 0.9, "0.90"));
			
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			L0A = L0;
			L1A = L1;
			L2A = L2;
			L3A = L3;
			L0 = (1 - gamma)*Close[0] + gamma*L0A;
			L1 = - gamma *L0 + L0A + gamma *L1A;
			L2 = - gamma *L1 + L1A + gamma *L2A;
			L3 = - gamma *L2 + L2A + gamma *L3A;
		
			CU = 0;
			CD = 0;
			
			if (L0 >= L1) 
				CU = L0 - L1; 
			else CD = L1 - L0;
			
			if (L1 >= L2) 
				CU = CU + L1 - L2; 
			else CD = CD + L2 - L1;
			
			if (L2 >= L3) 
				CU = CU + L2 - L3; 
			else CD = CD + L3 - L2;
		
			if (CU + CD != 0) 
				LRSI = CU / (CU + CD);
					
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
            Plot0.Set(LRSI);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("Gamma Smoothing")]
        [Category("Parameters")]
        public double Gamma
        {
            get { return gamma; }
            set { gamma = Math.Max(0.500, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Laguerre[] cacheLaguerre = null;

        private static Laguerre checkLaguerre = new Laguerre();

        /// <summary>
        /// Laguerre implementation
        /// </summary>
        /// <returns></returns>
        public Laguerre Laguerre(double gamma)
        {
            return Laguerre(Input, gamma);
        }

        /// <summary>
        /// Laguerre implementation
        /// </summary>
        /// <returns></returns>
        public Laguerre Laguerre(Data.IDataSeries input, double gamma)
        {
            checkLaguerre.Gamma = gamma;
            gamma = checkLaguerre.Gamma;

            if (cacheLaguerre != null)
                for (int idx = 0; idx < cacheLaguerre.Length; idx++)
                    if (Math.Abs(cacheLaguerre[idx].Gamma - gamma) <= double.Epsilon && cacheLaguerre[idx].EqualsInput(input))
                        return cacheLaguerre[idx];

            Laguerre indicator = new Laguerre();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Gamma = gamma;

            Laguerre[] tmp = new Laguerre[cacheLaguerre == null ? 1 : cacheLaguerre.Length + 1];
            if (cacheLaguerre != null)
                cacheLaguerre.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheLaguerre = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Laguerre implementation
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Laguerre Laguerre(double gamma)
        {
            return _indicator.Laguerre(Input, gamma);
        }

        /// <summary>
        /// Laguerre implementation
        /// </summary>
        /// <returns></returns>
        public Indicator.Laguerre Laguerre(Data.IDataSeries input, double gamma)
        {
            return _indicator.Laguerre(input, gamma);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Laguerre implementation
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Laguerre Laguerre(double gamma)
        {
            return _indicator.Laguerre(Input, gamma);
        }

        /// <summary>
        /// Laguerre implementation
        /// </summary>
        /// <returns></returns>
        public Indicator.Laguerre Laguerre(Data.IDataSeries input, double gamma)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Laguerre(input, gamma);
        }

    }
}
#endregion
